'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_direct_state_access'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_direct_state_access',error_checker=_errors._error_checker)
GL_PROGRAM_MATRIX_EXT=_C('GL_PROGRAM_MATRIX_EXT',0x8E2D)
GL_PROGRAM_MATRIX_STACK_DEPTH_EXT=_C('GL_PROGRAM_MATRIX_STACK_DEPTH_EXT',0x8E2F)
GL_TRANSPOSE_PROGRAM_MATRIX_EXT=_C('GL_TRANSPOSE_PROGRAM_MATRIX_EXT',0x8E2E)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint)
def glBindMultiTextureEXT(texunit,target,texture):pass
@_f
@_p.types(_cs.GLenum,_cs.GLuint,_cs.GLenum)
def glCheckNamedFramebufferStatusEXT(framebuffer,target):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glClearNamedBufferDataEXT(buffer,internalformat,format,type,data):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLsizeiptr,_cs.GLsizeiptr,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glClearNamedBufferSubDataEXT(buffer,internalformat,offset,size,format,type,data):pass
@_f
@_p.types(None,_cs.GLbitfield)
def glClientAttribDefaultEXT(mask):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLint,_cs.GLsizei,ctypes.c_void_p)
def glCompressedMultiTexImage1DEXT(texunit,target,level,internalformat,width,border,imageSize,bits):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLint,_cs.GLsizei,ctypes.c_void_p)
def glCompressedMultiTexImage2DEXT(texunit,target,level,internalformat,width,height,border,imageSize,bits):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLint,_cs.GLsizei,ctypes.c_void_p)
def glCompressedMultiTexImage3DEXT(texunit,target,level,internalformat,width,height,depth,border,imageSize,bits):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glCompressedMultiTexSubImage1DEXT(texunit,target,level,xoffset,width,format,imageSize,bits):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glCompressedMultiTexSubImage2DEXT(texunit,target,level,xoffset,yoffset,width,height,format,imageSize,bits):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glCompressedMultiTexSubImage3DEXT(texunit,target,level,xoffset,yoffset,zoffset,width,height,depth,format,imageSize,bits):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLint,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTextureImage1DEXT(texture,target,level,internalformat,width,border,imageSize,bits):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLint,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTextureImage2DEXT(texture,target,level,internalformat,width,height,border,imageSize,bits):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLint,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTextureImage3DEXT(texture,target,level,internalformat,width,height,depth,border,imageSize,bits):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTextureSubImage1DEXT(texture,target,level,xoffset,width,format,imageSize,bits):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTextureSubImage2DEXT(texture,target,level,xoffset,yoffset,width,height,format,imageSize,bits):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTextureSubImage3DEXT(texture,target,level,xoffset,yoffset,zoffset,width,height,depth,format,imageSize,bits):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLint)
def glCopyMultiTexImage1DEXT(texunit,target,level,internalformat,x,y,width,border):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLint)
def glCopyMultiTexImage2DEXT(texunit,target,level,internalformat,x,y,width,height,border):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei)
def glCopyMultiTexSubImage1DEXT(texunit,target,level,xoffset,x,y,width):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei)
def glCopyMultiTexSubImage2DEXT(texunit,target,level,xoffset,yoffset,x,y,width,height):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei)
def glCopyMultiTexSubImage3DEXT(texunit,target,level,xoffset,yoffset,zoffset,x,y,width,height):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLint)
def glCopyTextureImage1DEXT(texture,target,level,internalformat,x,y,width,border):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLint)
def glCopyTextureImage2DEXT(texture,target,level,internalformat,x,y,width,height,border):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei)
def glCopyTextureSubImage1DEXT(texture,target,level,xoffset,x,y,width):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei)
def glCopyTextureSubImage2DEXT(texture,target,level,xoffset,yoffset,x,y,width,height):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei)
def glCopyTextureSubImage3DEXT(texture,target,level,xoffset,yoffset,zoffset,x,y,width,height):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glDisableClientStateIndexedEXT(array,index):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glDisableClientStateiEXT(array,index):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glDisableIndexedEXT(target,index):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint)
def glDisableVertexArrayAttribEXT(vaobj,index):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum)
def glDisableVertexArrayEXT(vaobj,array):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glEnableClientStateIndexedEXT(array,index):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glEnableClientStateiEXT(array,index):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glEnableIndexedEXT(target,index):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint)
def glEnableVertexArrayAttribEXT(vaobj,index):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum)
def glEnableVertexArrayEXT(vaobj,array):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLintptr,_cs.GLsizeiptr)
def glFlushMappedNamedBufferRangeEXT(buffer,offset,length):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum)
def glFramebufferDrawBufferEXT(framebuffer,mode):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,arrays.GLuintArray)
def glFramebufferDrawBuffersEXT(framebuffer,n,bufs):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum)
def glFramebufferReadBufferEXT(framebuffer,mode):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum)
def glGenerateMultiTexMipmapEXT(texunit,target):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum)
def glGenerateTextureMipmapEXT(texture,target):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLbooleanArray)
def glGetBooleanIndexedvEXT(target,index,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,ctypes.c_void_p)
def glGetCompressedMultiTexImageEXT(texunit,target,lod,img):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,ctypes.c_void_p)
def glGetCompressedTextureImageEXT(texture,target,lod,img):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLdoubleArray)
def glGetDoubleIndexedvEXT(target,index,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLdoubleArray)
def glGetDoublei_vEXT(pname,index,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLfloatArray)
def glGetFloatIndexedvEXT(target,index,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLfloatArray)
def glGetFloati_vEXT(pname,index,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetFramebufferParameterivEXT(framebuffer,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLintArray)
def glGetIntegerIndexedvEXT(target,index,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glGetMultiTexEnvfvEXT(texunit,target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetMultiTexEnvivEXT(texunit,target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLdoubleArray)
def glGetMultiTexGendvEXT(texunit,coord,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glGetMultiTexGenfvEXT(texunit,coord,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetMultiTexGenivEXT(texunit,coord,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glGetMultiTexImageEXT(texunit,target,level,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLenum,arrays.GLfloatArray)
def glGetMultiTexLevelParameterfvEXT(texunit,target,level,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLenum,arrays.GLintArray)
def glGetMultiTexLevelParameterivEXT(texunit,target,level,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetMultiTexParameterIivEXT(texunit,target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLuintArray)
def glGetMultiTexParameterIuivEXT(texunit,target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glGetMultiTexParameterfvEXT(texunit,target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetMultiTexParameterivEXT(texunit,target,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetNamedBufferParameterivEXT(buffer,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLvoidpArray)
def glGetNamedBufferPointervEXT(buffer,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLintptr,_cs.GLsizeiptr,ctypes.c_void_p)
def glGetNamedBufferSubDataEXT(buffer,offset,size,data):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetNamedFramebufferAttachmentParameterivEXT(framebuffer,attachment,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetNamedFramebufferParameterivEXT(framebuffer,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,arrays.GLintArray)
def glGetNamedProgramLocalParameterIivEXT(program,target,index,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,arrays.GLuintArray)
def glGetNamedProgramLocalParameterIuivEXT(program,target,index,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,arrays.GLdoubleArray)
def glGetNamedProgramLocalParameterdvEXT(program,target,index,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,arrays.GLfloatArray)
def glGetNamedProgramLocalParameterfvEXT(program,target,index,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glGetNamedProgramStringEXT(program,target,pname,string):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetNamedProgramivEXT(program,target,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetNamedRenderbufferParameterivEXT(renderbuffer,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLvoidpArray)
def glGetPointerIndexedvEXT(target,index,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLvoidpArray)
def glGetPointeri_vEXT(pname,index,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glGetTextureImageEXT(texture,target,level,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLenum,arrays.GLfloatArray)
def glGetTextureLevelParameterfvEXT(texture,target,level,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLenum,arrays.GLintArray)
def glGetTextureLevelParameterivEXT(texture,target,level,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetTextureParameterIivEXT(texture,target,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,arrays.GLuintArray)
def glGetTextureParameterIuivEXT(texture,target,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glGetTextureParameterfvEXT(texture,target,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetTextureParameterivEXT(texture,target,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetVertexArrayIntegeri_vEXT(vaobj,index,pname,param):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetVertexArrayIntegervEXT(vaobj,pname,param):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,arrays.GLvoidpArray)
def glGetVertexArrayPointeri_vEXT(vaobj,index,pname,param):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLvoidpArray)
def glGetVertexArrayPointervEXT(vaobj,pname,param):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLenum,_cs.GLuint)
def glIsEnabledIndexedEXT(target,index):pass
@_f
@_p.types(ctypes.c_void_p,_cs.GLuint,_cs.GLenum)
def glMapNamedBufferEXT(buffer,access):pass
@_f
@_p.types(ctypes.c_void_p,_cs.GLuint,_cs.GLintptr,_cs.GLsizeiptr,_cs.GLbitfield)
def glMapNamedBufferRangeEXT(buffer,offset,length,access):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glMatrixFrustumEXT(mode,left,right,bottom,top,zNear,zFar):pass
@_f
@_p.types(None,_cs.GLenum)
def glMatrixLoadIdentityEXT(mode):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLdoubleArray)
def glMatrixLoadTransposedEXT(mode,m):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glMatrixLoadTransposefEXT(mode,m):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLdoubleArray)
def glMatrixLoaddEXT(mode,m):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glMatrixLoadfEXT(mode,m):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLdoubleArray)
def glMatrixMultTransposedEXT(mode,m):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glMatrixMultTransposefEXT(mode,m):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLdoubleArray)
def glMatrixMultdEXT(mode,m):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glMatrixMultfEXT(mode,m):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glMatrixOrthoEXT(mode,left,right,bottom,top,zNear,zFar):pass
@_f
@_p.types(None,_cs.GLenum)
def glMatrixPopEXT(mode):pass
@_f
@_p.types(None,_cs.GLenum)
def glMatrixPushEXT(mode):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glMatrixRotatedEXT(mode,angle,x,y,z):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glMatrixRotatefEXT(mode,angle,x,y,z):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glMatrixScaledEXT(mode,x,y,z):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glMatrixScalefEXT(mode,x,y,z):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glMatrixTranslatedEXT(mode,x,y,z):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glMatrixTranslatefEXT(mode,x,y,z):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLuint)
def glMultiTexBufferEXT(texunit,target,internalformat,buffer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glMultiTexCoordPointerEXT(texunit,size,type,stride,pointer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLfloat)
def glMultiTexEnvfEXT(texunit,target,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glMultiTexEnvfvEXT(texunit,target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLint)
def glMultiTexEnviEXT(texunit,target,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glMultiTexEnvivEXT(texunit,target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLdouble)
def glMultiTexGendEXT(texunit,coord,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLdoubleArray)
def glMultiTexGendvEXT(texunit,coord,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLfloat)
def glMultiTexGenfEXT(texunit,coord,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glMultiTexGenfvEXT(texunit,coord,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLint)
def glMultiTexGeniEXT(texunit,coord,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glMultiTexGenivEXT(texunit,coord,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLint,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glMultiTexImage1DEXT(texunit,target,level,internalformat,width,border,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLint,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glMultiTexImage2DEXT(texunit,target,level,internalformat,width,height,border,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLint,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glMultiTexImage3DEXT(texunit,target,level,internalformat,width,height,depth,border,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glMultiTexParameterIivEXT(texunit,target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLuintArray)
def glMultiTexParameterIuivEXT(texunit,target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLfloat)
def glMultiTexParameterfEXT(texunit,target,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glMultiTexParameterfvEXT(texunit,target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLint)
def glMultiTexParameteriEXT(texunit,target,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glMultiTexParameterivEXT(texunit,target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint)
def glMultiTexRenderbufferEXT(texunit,target,renderbuffer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glMultiTexSubImage1DEXT(texunit,target,level,xoffset,width,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glMultiTexSubImage2DEXT(texunit,target,level,xoffset,yoffset,width,height,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glMultiTexSubImage3DEXT(texunit,target,level,xoffset,yoffset,zoffset,width,height,depth,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizeiptr,ctypes.c_void_p,_cs.GLenum)
def glNamedBufferDataEXT(buffer,size,data,usage):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizeiptr,ctypes.c_void_p,_cs.GLbitfield)
def glNamedBufferStorageEXT(buffer,size,data,flags):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLintptr,_cs.GLsizeiptr,ctypes.c_void_p)
def glNamedBufferSubDataEXT(buffer,offset,size,data):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLintptr,_cs.GLintptr,_cs.GLsizeiptr)
def glNamedCopyBufferSubDataEXT(readBuffer,writeBuffer,readOffset,writeOffset,size):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint)
def glNamedFramebufferParameteriEXT(framebuffer,pname,param):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,_cs.GLuint)
def glNamedFramebufferRenderbufferEXT(framebuffer,attachment,renderbuffertarget,renderbuffer):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLint)
def glNamedFramebufferTexture1DEXT(framebuffer,attachment,textarget,texture,level):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLint)
def glNamedFramebufferTexture2DEXT(framebuffer,attachment,textarget,texture,level):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLint,_cs.GLint)
def glNamedFramebufferTexture3DEXT(framebuffer,attachment,textarget,texture,level,zoffset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLint)
def glNamedFramebufferTextureEXT(framebuffer,attachment,texture,level):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLint,_cs.GLenum)
def glNamedFramebufferTextureFaceEXT(framebuffer,attachment,texture,level,face):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLint,_cs.GLint)
def glNamedFramebufferTextureLayerEXT(framebuffer,attachment,texture,level,layer):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glNamedProgramLocalParameter4dEXT(program,target,index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,arrays.GLdoubleArray)
def glNamedProgramLocalParameter4dvEXT(program,target,index,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glNamedProgramLocalParameter4fEXT(program,target,index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,arrays.GLfloatArray)
def glNamedProgramLocalParameter4fvEXT(program,target,index,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint)
def glNamedProgramLocalParameterI4iEXT(program,target,index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,arrays.GLintArray)
def glNamedProgramLocalParameterI4ivEXT(program,target,index,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glNamedProgramLocalParameterI4uiEXT(program,target,index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,arrays.GLuintArray)
def glNamedProgramLocalParameterI4uivEXT(program,target,index,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLfloatArray)
def glNamedProgramLocalParameters4fvEXT(program,target,index,count,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLintArray)
def glNamedProgramLocalParametersI4ivEXT(program,target,index,count,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLuintArray)
def glNamedProgramLocalParametersI4uivEXT(program,target,index,count,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glNamedProgramStringEXT(program,target,format,len,string):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLsizei,_cs.GLsizei)
def glNamedRenderbufferStorageEXT(renderbuffer,internalformat,width,height):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei)
def glNamedRenderbufferStorageMultisampleCoverageEXT(renderbuffer,coverageSamples,colorSamples,internalformat,width,height):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei)
def glNamedRenderbufferStorageMultisampleEXT(renderbuffer,samples,internalformat,width,height):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLdouble)
def glProgramUniform1dEXT(program,location,x):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLdoubleArray)
def glProgramUniform1dvEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLfloat)
def glProgramUniform1fEXT(program,location,v0):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLfloatArray)
def glProgramUniform1fvEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint)
def glProgramUniform1iEXT(program,location,v0):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLintArray)
def glProgramUniform1ivEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint)
def glProgramUniform1uiEXT(program,location,v0):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuintArray)
def glProgramUniform1uivEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLdouble,_cs.GLdouble)
def glProgramUniform2dEXT(program,location,x,y):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLdoubleArray)
def glProgramUniform2dvEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLfloat,_cs.GLfloat)
def glProgramUniform2fEXT(program,location,v0,v1):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLfloatArray)
def glProgramUniform2fvEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLint)
def glProgramUniform2iEXT(program,location,v0,v1):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLintArray)
def glProgramUniform2ivEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint,_cs.GLuint)
def glProgramUniform2uiEXT(program,location,v0,v1):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuintArray)
def glProgramUniform2uivEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glProgramUniform3dEXT(program,location,x,y,z):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLdoubleArray)
def glProgramUniform3dvEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glProgramUniform3fEXT(program,location,v0,v1,v2):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLfloatArray)
def glProgramUniform3fvEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint)
def glProgramUniform3iEXT(program,location,v0,v1,v2):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLintArray)
def glProgramUniform3ivEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glProgramUniform3uiEXT(program,location,v0,v1,v2):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuintArray)
def glProgramUniform3uivEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glProgramUniform4dEXT(program,location,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLdoubleArray)
def glProgramUniform4dvEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glProgramUniform4fEXT(program,location,v0,v1,v2,v3):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLfloatArray)
def glProgramUniform4fvEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint)
def glProgramUniform4iEXT(program,location,v0,v1,v2,v3):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLintArray)
def glProgramUniform4ivEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glProgramUniform4uiEXT(program,location,v0,v1,v2,v3):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuintArray)
def glProgramUniform4uivEXT(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glProgramUniformMatrix2dvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glProgramUniformMatrix2fvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glProgramUniformMatrix2x3dvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glProgramUniformMatrix2x3fvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glProgramUniformMatrix2x4dvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glProgramUniformMatrix2x4fvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glProgramUniformMatrix3dvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glProgramUniformMatrix3fvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glProgramUniformMatrix3x2dvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glProgramUniformMatrix3x2fvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glProgramUniformMatrix3x4dvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glProgramUniformMatrix3x4fvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glProgramUniformMatrix4dvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glProgramUniformMatrix4fvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glProgramUniformMatrix4x2dvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glProgramUniformMatrix4x2fvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glProgramUniformMatrix4x3dvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glProgramUniformMatrix4x3fvEXT(program,location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLbitfield)
def glPushClientAttribDefaultEXT(mask):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,_cs.GLuint)
def glTextureBufferEXT(texture,target,internalformat,buffer):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLintptr,_cs.GLsizeiptr)
def glTextureBufferRangeEXT(texture,target,internalformat,buffer,offset,size):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLint,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glTextureImage1DEXT(texture,target,level,internalformat,width,border,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLint,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glTextureImage2DEXT(texture,target,level,internalformat,width,height,border,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLint,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glTextureImage3DEXT(texture,target,level,internalformat,width,height,depth,border,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLboolean)
def glTexturePageCommitmentEXT(texture,level,xoffset,yoffset,zoffset,width,height,depth,resident):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glTextureParameterIivEXT(texture,target,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,arrays.GLuintArray)
def glTextureParameterIuivEXT(texture,target,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,_cs.GLfloat)
def glTextureParameterfEXT(texture,target,pname,param):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glTextureParameterfvEXT(texture,target,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,_cs.GLint)
def glTextureParameteriEXT(texture,target,pname,param):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glTextureParameterivEXT(texture,target,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint)
def glTextureRenderbufferEXT(texture,target,renderbuffer):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei)
def glTextureStorage1DEXT(texture,target,levels,internalformat,width):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei)
def glTextureStorage2DEXT(texture,target,levels,internalformat,width,height):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLboolean)
def glTextureStorage2DMultisampleEXT(texture,target,samples,internalformat,width,height,fixedsamplelocations):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei)
def glTextureStorage3DEXT(texture,target,levels,internalformat,width,height,depth):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLboolean)
def glTextureStorage3DMultisampleEXT(texture,target,samples,internalformat,width,height,depth,fixedsamplelocations):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glTextureSubImage1DEXT(texture,target,level,xoffset,width,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glTextureSubImage2DEXT(texture,target,level,xoffset,yoffset,width,height,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glTextureSubImage3DEXT(texture,target,level,xoffset,yoffset,zoffset,width,height,depth,format,type,pixels):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glUnmapNamedBufferEXT(buffer):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLintptr,_cs.GLsizei)
def glVertexArrayBindVertexBufferEXT(vaobj,bindingindex,buffer,offset,stride):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLintptr)
def glVertexArrayColorOffsetEXT(vaobj,buffer,size,type,stride,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLsizei,_cs.GLintptr)
def glVertexArrayEdgeFlagOffsetEXT(vaobj,buffer,stride,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,_cs.GLsizei,_cs.GLintptr)
def glVertexArrayFogCoordOffsetEXT(vaobj,buffer,type,stride,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,_cs.GLsizei,_cs.GLintptr)
def glVertexArrayIndexOffsetEXT(vaobj,buffer,type,stride,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLintptr)
def glVertexArrayMultiTexCoordOffsetEXT(vaobj,buffer,texunit,size,type,stride,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,_cs.GLsizei,_cs.GLintptr)
def glVertexArrayNormalOffsetEXT(vaobj,buffer,type,stride,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLintptr)
def glVertexArraySecondaryColorOffsetEXT(vaobj,buffer,size,type,stride,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLintptr)
def glVertexArrayTexCoordOffsetEXT(vaobj,buffer,size,type,stride,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glVertexArrayVertexAttribBindingEXT(vaobj,attribindex,bindingindex):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glVertexArrayVertexAttribDivisorEXT(vaobj,index,divisor):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLboolean,_cs.GLuint)
def glVertexArrayVertexAttribFormatEXT(vaobj,attribindex,size,type,normalized,relativeoffset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLuint)
def glVertexArrayVertexAttribIFormatEXT(vaobj,attribindex,size,type,relativeoffset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLintptr)
def glVertexArrayVertexAttribIOffsetEXT(vaobj,buffer,index,size,type,stride,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLuint)
def glVertexArrayVertexAttribLFormatEXT(vaobj,attribindex,size,type,relativeoffset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLintptr)
def glVertexArrayVertexAttribLOffsetEXT(vaobj,buffer,index,size,type,stride,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLboolean,_cs.GLsizei,_cs.GLintptr)
def glVertexArrayVertexAttribOffsetEXT(vaobj,buffer,index,size,type,normalized,stride,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glVertexArrayVertexBindingDivisorEXT(vaobj,bindingindex,divisor):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLintptr)
def glVertexArrayVertexOffsetEXT(vaobj,buffer,size,type,stride,offset):pass



'''OpenGL extension ATI.texture_float

This module customises the behaviour of the 
OpenGL.raw.GL.ATI.texture_float to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds texture internal formats with 32 and 16 bit
	floating-point components.  The 32 bit floating-point components
	are in the standard IEEE float format.  The 16 bit floating-point
	components have 1 sign bit, 5 exponent bits, and 10 mantissa bits.
	Floating-point components are clamped to the limits of the range
	representable by their format.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ATI/texture_float.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ATI.texture_float import *
from OpenGL.raw.GL.ATI.texture_float import _EXTENSION_NAME

def glInitTextureFloatATI():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.visual_info

This module customises the behaviour of the 
OpenGL.raw.GLX.EXT.visual_info to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows the user to request a particular X visual 
	type to be associated with a GLX visual, and allows the user
	to query the X visual type underlying a GLX visual.
	
	In addition, this extension provides a means to request a visual
	with a transparent pixel and to query whether a visual supports a
	transparent pixel value and the value of the transparent pixel.
	Note that the notion of level and transparent pixels are orthogonal as
	both layer 1 and layer 0 visuals may or may not have a transparent pixel
	value.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/visual_info.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLX import _types, _glgets
from OpenGL.raw.GLX.EXT.visual_info import *
from OpenGL.raw.GLX.EXT.visual_info import _EXTENSION_NAME

def glInitVisualInfoEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.packed_pixels

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.packed_pixels to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides support for packed pixels in host memory.  A
	packed pixel is represented entirely by one unsigned byte, one
	unsigned short, or one unsigned integer.  The fields with the packed
	pixel are not proper machine types, but the pixel as a whole is.  Thus
	the pixel storage modes, including PACK_SKIP_PIXELS, PACK_ROW_LENGTH,
	PACK_SKIP_ROWS, PACK_IMAGE_HEIGHT_EXT, PACK_SKIP_IMAGES_EXT,
	PACK_SWAP_BYTES, PACK_ALIGNMENT, and their unpacking counterparts all
	work correctly with packed pixels.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/packed_pixels.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.packed_pixels import *
from OpenGL.raw.GL.EXT.packed_pixels import _EXTENSION_NAME

def glInitPackedPixelsEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension INGR.color_clamp

This module customises the behaviour of the 
OpenGL.raw.GL.INGR.color_clamp to provide a more 
Python-friendly API

Overview (from the spec)
	
	Various RGBA color space conversions require clamping to values
	in a more constrained range than [0, 1].  This extension allows
	the definition of independent color clamp values for each of the
	four color components as part of the Final Conversion in the pixel
	transfer path for draws, reads, and copies.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/INGR/color_clamp.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.INGR.color_clamp import *
from OpenGL.raw.GL.INGR.color_clamp import _EXTENSION_NAME

def glInitColorClampINGR():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_NV_coverage_sample'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_NV_coverage_sample',error_checker=_errors._error_checker)
GL_COVERAGE_ALL_FRAGMENTS_NV=_C('GL_COVERAGE_ALL_FRAGMENTS_NV',0x8ED5)
GL_COVERAGE_ATTACHMENT_NV=_C('GL_COVERAGE_ATTACHMENT_NV',0x8ED2)
GL_COVERAGE_AUTOMATIC_NV=_C('GL_COVERAGE_AUTOMATIC_NV',0x8ED7)
GL_COVERAGE_BUFFERS_NV=_C('GL_COVERAGE_BUFFERS_NV',0x8ED3)
GL_COVERAGE_BUFFER_BIT_NV=_C('GL_COVERAGE_BUFFER_BIT_NV',0x00008000)
GL_COVERAGE_COMPONENT4_NV=_C('GL_COVERAGE_COMPONENT4_NV',0x8ED1)
GL_COVERAGE_COMPONENT_NV=_C('GL_COVERAGE_COMPONENT_NV',0x8ED0)
GL_COVERAGE_EDGE_FRAGMENTS_NV=_C('GL_COVERAGE_EDGE_FRAGMENTS_NV',0x8ED6)
GL_COVERAGE_SAMPLES_NV=_C('GL_COVERAGE_SAMPLES_NV',0x8ED4)
@_f
@_p.types(None,_cs.GLboolean)
def glCoverageMaskNV(mask):pass
@_f
@_p.types(None,_cs.GLenum)
def glCoverageOperationNV(operation):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_APPLE_fence'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_APPLE_fence',error_checker=_errors._error_checker)
GL_DRAW_PIXELS_APPLE=_C('GL_DRAW_PIXELS_APPLE',0x8A0A)
GL_FENCE_APPLE=_C('GL_FENCE_APPLE',0x8A0B)
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glDeleteFencesAPPLE(n,fences):pass
@_f
@_p.types(None,_cs.GLuint)
def glFinishFenceAPPLE(fence):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint)
def glFinishObjectAPPLE(object,name):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glGenFencesAPPLE(n,fences):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glIsFenceAPPLE(fence):pass
@_f
@_p.types(None,_cs.GLuint)
def glSetFenceAPPLE(fence):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glTestFenceAPPLE(fence):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLenum,_cs.GLuint)
def glTestObjectAPPLE(object,name):pass



'''OpenGL extension EXT.convolution

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.convolution to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines 1 and 2 dimensional convolution operations
	at a fixed location in the pixel transfer process.  Thus pixel drawing,
	reading, and copying, as well as texture image definition, are all
	candidates for convolution.  The convolution kernels are themselves
	treated as 1 and 2 dimensional images, which can be loaded from
	application memory or from the framebuffer.
	
	This extension is designed to accommodate 3D convolution, but the
	API is left for a future extension.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/convolution.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.convolution import *
from OpenGL.raw.GL.EXT.convolution import _EXTENSION_NAME

def glInitConvolutionEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glConvolutionFilter1DEXT.image size not checked against 'format,type,width'
glConvolutionFilter1DEXT=wrapper.wrapper(glConvolutionFilter1DEXT).setInputArraySize(
    'image', None
)
# INPUT glConvolutionFilter2DEXT.image size not checked against 'format,type,width,height'
glConvolutionFilter2DEXT=wrapper.wrapper(glConvolutionFilter2DEXT).setInputArraySize(
    'image', None
)
# INPUT glConvolutionParameterfvEXT.params size not checked against 'pname'
glConvolutionParameterfvEXT=wrapper.wrapper(glConvolutionParameterfvEXT).setInputArraySize(
    'params', None
)
# INPUT glConvolutionParameterivEXT.params size not checked against 'pname'
glConvolutionParameterivEXT=wrapper.wrapper(glConvolutionParameterivEXT).setInputArraySize(
    'params', None
)
# OUTPUT glGetConvolutionFilterEXT.image COMPSIZE(target, format, type) 
glGetConvolutionParameterfvEXT=wrapper.wrapper(glGetConvolutionParameterfvEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetConvolutionParameterivEXT=wrapper.wrapper(glGetConvolutionParameterivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# OUTPUT glGetSeparableFilterEXT.column COMPSIZE(target, format, type) 
# OUTPUT glGetSeparableFilterEXT.row COMPSIZE(target, format, type) 
# OUTPUT glGetSeparableFilterEXT.span COMPSIZE(target, format, type) 
# INPUT glSeparableFilter2DEXT.column size not checked against 'target,format,type,height'
# INPUT glSeparableFilter2DEXT.row size not checked against 'target,format,type,width'
glSeparableFilter2DEXT=wrapper.wrapper(glSeparableFilter2DEXT).setInputArraySize(
    'column', None
).setInputArraySize(
    'row', None
)
### END AUTOGENERATED SECTION


"""Extension module support methods

This module provides the tools required to check whether
an extension is available
"""
from OpenGL.latebind import LateBind
from OpenGL._bytes import bytes,unicode,as_8_bit
import OpenGL as root
import sys
import logging
_log = logging.getLogger( 'OpenGL.extensions' )
VERSION_PREFIX = as_8_bit('GL_VERSION_GL_')
CURRENT_GL_VERSION = None
AVAILABLE_GL_EXTENSIONS = []
AVAILABLE_GLU_EXTENSIONS = []

# version tuple -> list of implicitly included extensions...
VERSION_EXTENSIONS = [
    ((3,0), [
        as_8_bit('GL_ARB_vertex_array_object'),
        as_8_bit('GL_ARB_texture_buffer_object'),
        as_8_bit('GL_ARB_framebuffer_object'),
        as_8_bit('GL_ARB_map_buffer_range'),
    ]),
    ((3,1), [
        as_8_bit('GL_ARB_copy_buffer'),
        as_8_bit('GL_ARB_uniform_buffer_object'),
    ]),
    ((3,2), [
        as_8_bit('GL_ARB_draw_elements_base_vertex'),
        as_8_bit('GL_ARB_provoking_vertex'),
        as_8_bit('GL_ARB_sync'),
        as_8_bit('GL_ARB_texture_multisample'),
    ]),
    ((3,3), [
        as_8_bit('GL_ARB_texture_multisample'),
        as_8_bit('GL_ARB_blend_func_extended'),
        as_8_bit('GL_ARB_sampler_objects'),
        as_8_bit('GL_ARB_explicit_attrib_location'),
        as_8_bit('GL_ARB_occlusion_query2'),
        as_8_bit('GL_ARB_shader_bit_encoding'),
        as_8_bit('GL_ARB_texture_rgb10_a2ui'),
        as_8_bit('GL_ARB_texture_swizzle'),
        as_8_bit('GL_ARB_timer_query'),
        as_8_bit('GL_ARB_vertex_type_2_10_10_10_rev'),
    ]),
    ((4,0), [
        as_8_bit('GL_ARB_texture_query_lod'),
        as_8_bit('GL_ARB_draw_indirect'),
        as_8_bit('GL_ARB_gpu_shader5'),
        as_8_bit('GL_ARB_gpu_shader_fp64'),
        as_8_bit('GL_ARB_shader_subroutine'),
        as_8_bit('GL_ARB_tessellation_shader'),
        as_8_bit('GL_ARB_texture_buffer_object_rgb32'),
        as_8_bit('GL_ARB_texture_cube_map_array'),
        as_8_bit('GL_ARB_texture_gather'),
        as_8_bit('GL_ARB_transform_feedback2'),
        as_8_bit('GL_ARB_transform_feedback3'),
    ]),
    ((4,1), [
        as_8_bit('GL_ARB_ES2_compatibility'),
        as_8_bit('GL_ARB_get_program_binary'),
        as_8_bit('GL_ARB_separate_shader_objects'),
        as_8_bit('GL_ARB_shader_precision'),
        as_8_bit('GL_ARB_vertex_attrib_64bit'),
        as_8_bit('GL_ARB_viewport_array'),
    ]),
    ((4,2), [
        as_8_bit('GL_ARB_base_instance'),
        as_8_bit('GL_ARB_shading_language_420pack'),
        as_8_bit('GL_ARB_transform_feedback_instanced'),
        as_8_bit('GL_ARB_compressed_texture_pixel_storage'),
        as_8_bit('GL_ARB_conservative_depth'),
        as_8_bit('GL_ARB_internalformat_query'),
        as_8_bit('GL_ARB_map_buffer_alignment'),
        as_8_bit('GL_ARB_shader_atomic_counters'),
        as_8_bit('GL_ARB_shader_image_load_store'),
        as_8_bit('GL_ARB_shading_language_packing'),
        as_8_bit('GL_ARB_texture_storage'),
    ]),
]

class ExtensionQuerier( object ):
    prefix = None
    version_prefix = None
    assumed_version = [1,0]
    
    version = extensions = None
    version_string = extensions_string = None
    
    registered = []
    def __init__( self ):
        self.registered.append( self )
    
    @classmethod 
    def hasExtension( self, specifier ):
        for registered in self.registered:
            result = registered( specifier )
            if result:
                return result 
        return False
    
    def __call__( self, specifier ):
        specifier = as_8_bit(specifier).replace(as_8_bit('.'),as_8_bit('_'))
        if not specifier.startswith( self.prefix ):
            return None 
        
        if specifier.startswith( self.version_prefix ):
            specifier = [
                int(x)
                for x in specifier[ len(self.version_prefix):].split(as_8_bit('_'))
            ]
            if specifier[:2] <= self.assumed_version:
                return True
            version = self.getVersion()
            if not version:
                return version
            return specifier <= version
        else:
            extensions = self.getExtensions()
            return extensions and specifier in extensions
    def getVersion( self ):
        if not self.version:
            self.version = self.pullVersion()
        return self.version 
    def getExtensions( self ):
        if not self.extensions:
            self.extensions = self.pullExtensions()
        return self.extensions

class _GLQuerier( ExtensionQuerier ):
    prefix = as_8_bit('GL_')
    version_prefix = as_8_bit('GL_VERSION_GL_')
    assumed_version = [1,1]
    def pullVersion( self ):
        """Retrieve 2-int declaration of major/minor GL version

        returns [int(major),int(minor)] or False if not loaded
        """
        from OpenGL import platform
        if not platform.PLATFORM.CurrentContextIsValid():
            return False
        from OpenGL.raw.GL.VERSION.GL_1_1 import glGetString 
        from OpenGL.raw.GL.VERSION.GL_1_1 import GL_VERSION
        new = glGetString( GL_VERSION )
        
        self.version_string = new
        if new:
            return [
                int(x) for x in new.split(as_8_bit(' '),1)[0].split( as_8_bit('.') )
            ]
        else:
            return False # not yet loaded/supported
    def pullExtensions( self ):
        from OpenGL import platform
        if not platform.PLATFORM.CurrentContextIsValid():
            return False
        from OpenGL.raw.GL._types import GLint
        from OpenGL.raw.GL.VERSION.GL_1_1 import glGetString, glGetError
        from OpenGL.raw.GL.VERSION.GL_1_1 import GL_EXTENSIONS
        from OpenGL import error
        try:
            extensions = glGetString( GL_EXTENSIONS )
            if glGetError():
                raise error.GLError()
            if extensions:
                extensions = extensions.split()
            else:
                return False
        except (AttributeError, error.GLError) as err:
            # OpenGL 3.0 deprecates glGetString( GL_EXTENSIONS )
            from OpenGL.raw.GL.VERSION.GL_3_0 import GL_NUM_EXTENSIONS, glGetStringi
            from OpenGL.raw.GL.VERSION.GL_1_1 import glGetIntegerv
            count = GLint()
            glGetIntegerv( GL_NUM_EXTENSIONS, count )
            extensions = []
            for i in range( count.value ):
                extension = glGetStringi( GL_EXTENSIONS, i )
                extensions.append(
                    extension
                )
        # Add included-by-reference extensions...
        version = self.getVersion()
        if not version:
            # should not be possible?
            return version 
        check = tuple( version[:2] )
        for (v,v_exts) in VERSION_EXTENSIONS:
            if v <= check:
                for v_ext in v_exts:
                    if v_ext not in extensions:
                        extensions.append( as_8_bit(v_ext) )
            else:
                break
        return extensions
GLQuerier = _GLQuerier()
class _GLUQuerier( ExtensionQuerier ):
    prefix = as_8_bit('GLU_')
    version_prefix = as_8_bit('GLU_VERSION_GL_')
    def pullVersion( self ):
        from OpenGL.GLU import gluGetString,GLU_VERSION
        return [
            int(x) for x in gluGetString( GLU_VERSION ).split('_')
            if x.isdigit()
        ]
    def pullExtensions( self ):
        from OpenGL.GLU import gluGetString,GLU_EXTENSIONS
        return gluGetString( GLU_EXTENSIONS ).split()
GLUQuerier = _GLUQuerier()

def hasExtension( specifier ):
    return ExtensionQuerier.hasExtension( specifier )
hasGLExtension = hasGLUExtension = hasExtension

class _Alternate( LateBind ):
    def __init__( self, name, *alternates ):
        """Initialize set of alternative implementations of the same function"""
        self.__name__ = name
        self._alternatives = alternates
        if root.MODULE_ANNOTATIONS:
            frame = sys._getframe().f_back
            if frame and frame.f_back and '__name__' in frame.f_back.f_globals:
                self.__module__ = frame.f_back.f_globals['__name__']
    def __bool__( self ):
        from OpenGL import error
        try:
            return bool( self.getFinalCall())
        except error.NullFunctionError as err:
            return False
    __nonzero__ = __bool__ # Python 2.6 compatibility
    def finalise( self ):
        """Call, doing a late lookup and bind to find an implementation"""
        for alternate in self._alternatives:
            if alternate:
#                _log.info(
#                    """Chose alternate: %s from %s""",
#                    alternate.__name__,
#                    ", ".join([x.__name__ for x in self._alternatives])
#                )
                return alternate
        from OpenGL import error
        raise error.NullFunctionError(
            """Attempt to call an undefined alternate function (%s), check for bool(%s) before calling"""%(
                ', '.join([x.__name__ for x in self._alternatives]),
                self.__name__,
            )
        )
def alternate( name, *functions ):
    """Construct a callable that functions as the first implementation found of given set of alternatives

    if name is a function then its name will be used....
    """
    if not isinstance( name, (bytes,unicode)):
        functions = (name,)+functions
        name = name.__name__
    return type( name, (_Alternate,), {} )( name, *functions )



'''OpenGL extension EXT.subtexture

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.subtexture to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows a contiguous portion of an already-existing
	texture image to be redefined, without affecting the remaining portion
	of the image, or any of the other state that describe the texture.  No
	provision is made to query a subregion of a texture.
	
	Semantics for null image pointers are defined for TexImage1D,
	TexImage2D, and TexImage3DEXT.  Null image pointers can be used by
	applications to effectively support texture arrays whose dimensions
	are not a power of 2.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/subtexture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.subtexture import *
from OpenGL.raw.GL.EXT.subtexture import _EXTENSION_NAME

def glInitSubtextureEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glTexSubImage1DEXT.pixels size not checked against 'format,type,width'
glTexSubImage1DEXT=wrapper.wrapper(glTexSubImage1DEXT).setInputArraySize(
    'pixels', None
)
# INPUT glTexSubImage2DEXT.pixels size not checked against 'format,type,width,height'
glTexSubImage2DEXT=wrapper.wrapper(glTexSubImage2DEXT).setInputArraySize(
    'pixels', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension NV.deep_texture3D

This module customises the behaviour of the 
OpenGL.raw.GL.NV.deep_texture3D to provide a more 
Python-friendly API

Overview (from the spec)
	
	Some applications require 3D textures that have a significant number of
	slices, but less resolution in width and height. In the current spec, the
	maximum value for the size of all three dimensions is specified by a
	single value. This extension adds a second set of limits against which 3D
	textures can be checked if an application needs deeper textures than would
	be allowed by the symmetric texture limits.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/deep_texture3D.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.deep_texture3D import *
from OpenGL.raw.GL.NV.deep_texture3D import _EXTENSION_NAME

def glInitDeepTexture3DNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_explicit_multisample'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_explicit_multisample',error_checker=_errors._error_checker)
GL_INT_SAMPLER_RENDERBUFFER_NV=_C('GL_INT_SAMPLER_RENDERBUFFER_NV',0x8E57)
GL_MAX_SAMPLE_MASK_WORDS_NV=_C('GL_MAX_SAMPLE_MASK_WORDS_NV',0x8E59)
GL_SAMPLER_RENDERBUFFER_NV=_C('GL_SAMPLER_RENDERBUFFER_NV',0x8E56)
GL_SAMPLE_MASK_NV=_C('GL_SAMPLE_MASK_NV',0x8E51)
GL_SAMPLE_MASK_VALUE_NV=_C('GL_SAMPLE_MASK_VALUE_NV',0x8E52)
GL_SAMPLE_POSITION_NV=_C('GL_SAMPLE_POSITION_NV',0x8E50)
GL_TEXTURE_BINDING_RENDERBUFFER_NV=_C('GL_TEXTURE_BINDING_RENDERBUFFER_NV',0x8E53)
GL_TEXTURE_RENDERBUFFER_DATA_STORE_BINDING_NV=_C('GL_TEXTURE_RENDERBUFFER_DATA_STORE_BINDING_NV',0x8E54)
GL_TEXTURE_RENDERBUFFER_NV=_C('GL_TEXTURE_RENDERBUFFER_NV',0x8E55)
GL_UNSIGNED_INT_SAMPLER_RENDERBUFFER_NV=_C('GL_UNSIGNED_INT_SAMPLER_RENDERBUFFER_NV',0x8E58)
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLfloatArray)
def glGetMultisamplefvNV(pname,index,val):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLbitfield)
def glSampleMaskIndexedNV(index,mask):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glTexRenderbufferNV(target,renderbuffer):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_OES_compressed_paletted_texture'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_OES_compressed_paletted_texture',error_checker=_errors._error_checker)
GL_PALETTE4_R5_G6_B5_OES=_C('GL_PALETTE4_R5_G6_B5_OES',0x8B92)
GL_PALETTE4_RGB5_A1_OES=_C('GL_PALETTE4_RGB5_A1_OES',0x8B94)
GL_PALETTE4_RGB8_OES=_C('GL_PALETTE4_RGB8_OES',0x8B90)
GL_PALETTE4_RGBA4_OES=_C('GL_PALETTE4_RGBA4_OES',0x8B93)
GL_PALETTE4_RGBA8_OES=_C('GL_PALETTE4_RGBA8_OES',0x8B91)
GL_PALETTE8_R5_G6_B5_OES=_C('GL_PALETTE8_R5_G6_B5_OES',0x8B97)
GL_PALETTE8_RGB5_A1_OES=_C('GL_PALETTE8_RGB5_A1_OES',0x8B99)
GL_PALETTE8_RGB8_OES=_C('GL_PALETTE8_RGB8_OES',0x8B95)
GL_PALETTE8_RGBA4_OES=_C('GL_PALETTE8_RGBA4_OES',0x8B98)
GL_PALETTE8_RGBA8_OES=_C('GL_PALETTE8_RGBA8_OES',0x8B96)




'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_MESA_window_pos'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_MESA_window_pos',error_checker=_errors._error_checker)

@_f
@_p.types(None,_cs.GLdouble,_cs.GLdouble)
def glWindowPos2dMESA(x,y):pass
@_f
@_p.types(None,arrays.GLdoubleArray)
def glWindowPos2dvMESA(v):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat)
def glWindowPos2fMESA(x,y):pass
@_f
@_p.types(None,arrays.GLfloatArray)
def glWindowPos2fvMESA(v):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint)
def glWindowPos2iMESA(x,y):pass
@_f
@_p.types(None,arrays.GLintArray)
def glWindowPos2ivMESA(v):pass
@_f
@_p.types(None,_cs.GLshort,_cs.GLshort)
def glWindowPos2sMESA(x,y):pass
@_f
@_p.types(None,arrays.GLshortArray)
def glWindowPos2svMESA(v):pass
@_f
@_p.types(None,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glWindowPos3dMESA(x,y,z):pass
@_f
@_p.types(None,arrays.GLdoubleArray)
def glWindowPos3dvMESA(v):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glWindowPos3fMESA(x,y,z):pass
@_f
@_p.types(None,arrays.GLfloatArray)
def glWindowPos3fvMESA(v):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint)
def glWindowPos3iMESA(x,y,z):pass
@_f
@_p.types(None,arrays.GLintArray)
def glWindowPos3ivMESA(v):pass
@_f
@_p.types(None,_cs.GLshort,_cs.GLshort,_cs.GLshort)
def glWindowPos3sMESA(x,y,z):pass
@_f
@_p.types(None,arrays.GLshortArray)
def glWindowPos3svMESA(v):pass
@_f
@_p.types(None,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glWindowPos4dMESA(x,y,z,w):pass
@_f
@_p.types(None,arrays.GLdoubleArray)
def glWindowPos4dvMESA(v):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glWindowPos4fMESA(x,y,z,w):pass
@_f
@_p.types(None,arrays.GLfloatArray)
def glWindowPos4fvMESA(v):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint)
def glWindowPos4iMESA(x,y,z,w):pass
@_f
@_p.types(None,arrays.GLintArray)
def glWindowPos4ivMESA(v):pass
@_f
@_p.types(None,_cs.GLshort,_cs.GLshort,_cs.GLshort,_cs.GLshort)
def glWindowPos4sMESA(x,y,z,w):pass
@_f
@_p.types(None,arrays.GLshortArray)
def glWindowPos4svMESA(v):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_sparse_buffer'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_sparse_buffer',error_checker=_errors._error_checker)
GL_SPARSE_BUFFER_PAGE_SIZE_ARB=_C('GL_SPARSE_BUFFER_PAGE_SIZE_ARB',0x82F8)
GL_SPARSE_STORAGE_BIT_ARB=_C('GL_SPARSE_STORAGE_BIT_ARB',0x0400)
@_f
@_p.types(None,_cs.GLenum,_cs.GLintptr,_cs.GLsizeiptr,_cs.GLboolean)
def glBufferPageCommitmentARB(target,offset,size,commit):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLintptr,_cs.GLsizeiptr,_cs.GLboolean)
def glNamedBufferPageCommitmentARB(buffer,offset,size,commit):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLintptr,_cs.GLsizeiptr,_cs.GLboolean)
def glNamedBufferPageCommitmentEXT(buffer,offset,size,commit):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.EGL import _types as _cs
# End users want this...
from OpenGL.raw.EGL._types import *
from OpenGL.raw.EGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'EGL_KHR_create_context'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.EGL,'EGL_KHR_create_context',error_checker=_errors._error_checker)
EGL_CONTEXT_FLAGS_KHR=_C('EGL_CONTEXT_FLAGS_KHR',0x30FC)
EGL_CONTEXT_MAJOR_VERSION_KHR=_C('EGL_CONTEXT_MAJOR_VERSION_KHR',0x3098)
EGL_CONTEXT_MINOR_VERSION_KHR=_C('EGL_CONTEXT_MINOR_VERSION_KHR',0x30FB)
EGL_CONTEXT_OPENGL_COMPATIBILITY_PROFILE_BIT_KHR=_C('EGL_CONTEXT_OPENGL_COMPATIBILITY_PROFILE_BIT_KHR',0x00000002)
EGL_CONTEXT_OPENGL_CORE_PROFILE_BIT_KHR=_C('EGL_CONTEXT_OPENGL_CORE_PROFILE_BIT_KHR',0x00000001)
EGL_CONTEXT_OPENGL_DEBUG_BIT_KHR=_C('EGL_CONTEXT_OPENGL_DEBUG_BIT_KHR',0x00000001)
EGL_CONTEXT_OPENGL_FORWARD_COMPATIBLE_BIT_KHR=_C('EGL_CONTEXT_OPENGL_FORWARD_COMPATIBLE_BIT_KHR',0x00000002)
EGL_CONTEXT_OPENGL_PROFILE_MASK_KHR=_C('EGL_CONTEXT_OPENGL_PROFILE_MASK_KHR',0x30FD)
EGL_CONTEXT_OPENGL_RESET_NOTIFICATION_STRATEGY_KHR=_C('EGL_CONTEXT_OPENGL_RESET_NOTIFICATION_STRATEGY_KHR',0x31BD)
EGL_CONTEXT_OPENGL_ROBUST_ACCESS_BIT_KHR=_C('EGL_CONTEXT_OPENGL_ROBUST_ACCESS_BIT_KHR',0x00000004)
EGL_LOSE_CONTEXT_ON_RESET_KHR=_C('EGL_LOSE_CONTEXT_ON_RESET_KHR',0x31BF)
EGL_NO_RESET_NOTIFICATION_KHR=_C('EGL_NO_RESET_NOTIFICATION_KHR',0x31BE)
EGL_OPENGL_ES3_BIT=_C('EGL_OPENGL_ES3_BIT',0x00000040)
EGL_OPENGL_ES3_BIT_KHR=_C('EGL_OPENGL_ES3_BIT_KHR',0x00000040)




'''OpenGL extension EXT.point_parameters

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.point_parameters to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension supports additional geometric characteristics of points. It
	can be used to render particles or tiny light sources, commonly referred
	as "Light points".
	
	The raster brightness of a point is a function of the point area, point
	color, point transparency, and the response of the display's electron gun
	and phosphor. The point area and the point transparency are derived from the
	point size, currently provided with the <size> parameter of glPointSize.
	
	The primary motivation is to allow the size of a point to be affected by
	distance attenuation. When distance attenuation has an effect, the final
	point size decreases as the distance of the point from the eye increases.
	
	The secondary motivation is a mean to control the mapping from the point
	size to the raster point area and point transparency. This is done in order
	to increase the dynamic range of the raster brightness of points. In other
	words, the alpha component of a point may be decreased (and its transparency
	increased) as its area shrinks below a defined threshold.
	
	This extension defines a derived point size to be closely related to point
	brightness. The brightness of a point is given by:
	
	                    1
	    dist_atten(d) = -------------------
	                    a + b * d + c * d^2
	
	    brightness(Pe) = Brightness * dist_atten(|Pe|)
	
	where 'Pe' is the point in eye coordinates, and 'Brightness' is some initial
	value proportional to the square of the size provided with glPointSize. Here
	we simplify the raster brightness to be a function of the rasterized point
	area and point transparency.
	
	                brightness(Pe)                      brightness(Pe) >= Threshold_Area
	    area(Pe) =
	                Threshold_Area                      Otherwise
	
	    factor(Pe) = brightness(Pe)/Threshold_Area
	
	    alpha(Pe) = Alpha * factor(Pe)
	
	where 'Alpha' comes with the point color (possibly modified by lighting).
	
	'Threshold_Area' above is in area units. Thus, it is proportional to the
	square of the threshold provided by the programmer through this extension.
	
	The new point size derivation method applies to all points, while the
	threshold applies to multisample points only.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/point_parameters.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.point_parameters import *
from OpenGL.raw.GL.EXT.point_parameters import _EXTENSION_NAME

def glInitPointParametersEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glPointParameterfvEXT.params size not checked against 'pname'
glPointParameterfvEXT=wrapper.wrapper(glPointParameterfvEXT).setInputArraySize(
    'params', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension ARB.texture_stencil8

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.texture_stencil8 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension accepts STENCIL_INDEX8 as a texture internal format, and
	adds STENCIL_INDEX8 to the required internal format list. This removes the
	need to use renderbuffers if a stencil-only format is desired.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/texture_stencil8.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.texture_stencil8 import *
from OpenGL.raw.GL.ARB.texture_stencil8 import _EXTENSION_NAME

def glInitTextureStencil8ARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension SGIS.blended_overlay

This module customises the behaviour of the 
OpenGL.raw.GLX.SGIS.blended_overlay to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension augments the set of transparency types for GLX
	visuals (defined by the EXT_visual_info GLX extension).  A new
	transparency type designated BLENDED_RGBA_SGIS is defined for
	overlay windows supporting an alpha color component.  Instead of
	using a particular transparent pixel or color value to indicate
	transparency, a window created with a BLENDED_RGBA_SGIS
	transparency type visual blends with the lower frame buffer layers
	based on the overlay window's per-pixel alpha component.
	
	The overlay blend function is:
	
	  if ( Cu is color index pixel type ) then
	
	    if ( Ao > 0.0 ) then
	      Cd = Co    /* Non-zero overlay alpha simply uses overlay color */
	    else
	      Cd = Cu    /* Note: later CI cmap turns Cd index into true color */
	    endif
	
	  else    /* Cu is an RGBA, RGB, L, or LA pixel type */
	
	    /* blend overlay color with normal color based on overlay alpha */
	    Cd = Co * (Ao,Ao,Ao,Ao) + Cu * ( (1,1,1,1) - (Ao,Ao,Ao,Ao) )
	
	  endif
	
	where Cd is the resulting displayed color.  Co is the RGBA
	quadruplet for the overlay RGBA components of the overlay window
	pixel.  Ao is the alpha component for the overlay alpha component
	of the overlay window pixel.  Cu is the RGBA quadruplet for the
	RGBA components of the displayed pixel if no overlay where present
	in the overlay window's layer (or any higher level).  Think of Cu
	as the displayable color "under" the BLENDED_RGBA_SGIS overlay
	pixel.
	
	In the case of extended range frame buffer formats, the overlay
	blend function is applied post-clamping to the [0,1] range.
	
	Here are some of the uses for blended RGBA overlays:
	
	  Render antialiased lines, points, and polygons in the overlays
	  with GL_LINE_SMOOTH, GL_POINT_SMOOTH, and GL_POLYGON_SMOOTH.
	
	  Alpha blended antiliasing could be used on the overlay boundaries
	  of overlay control panels and boat hulls.
	
	  Clean blending of overlaid mattes for live video.  Live video
	  stream could be overlaid by computer generated overlaid
	  graphics.
	
	  Swank user interface support for sweeping out irregular screen
	  regions by painiting the region in the overlay with a fractional
	  alpha to blend the sweeped region with some constant color in the
	  overlay.
	
	  Swank brush shapes for 3D painting that let you see the blended
	  pixels underneather the overlay brush shape.
	
	  Really smooth (ie, subpixel positioned), full-color, antialiased
	  cursors (application drawn into a blended RGBA overlay).

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIS/blended_overlay.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLX import _types, _glgets
from OpenGL.raw.GLX.SGIS.blended_overlay import *
from OpenGL.raw.GLX.SGIS.blended_overlay import _EXTENSION_NAME

def glInitBlendedOverlaySGIS():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_window_pos'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_window_pos',error_checker=_errors._error_checker)

@_f
@_p.types(None,_cs.GLdouble,_cs.GLdouble)
def glWindowPos2dARB(x,y):pass
@_f
@_p.types(None,arrays.GLdoubleArray)
def glWindowPos2dvARB(v):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat)
def glWindowPos2fARB(x,y):pass
@_f
@_p.types(None,arrays.GLfloatArray)
def glWindowPos2fvARB(v):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint)
def glWindowPos2iARB(x,y):pass
@_f
@_p.types(None,arrays.GLintArray)
def glWindowPos2ivARB(v):pass
@_f
@_p.types(None,_cs.GLshort,_cs.GLshort)
def glWindowPos2sARB(x,y):pass
@_f
@_p.types(None,arrays.GLshortArray)
def glWindowPos2svARB(v):pass
@_f
@_p.types(None,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glWindowPos3dARB(x,y,z):pass
@_f
@_p.types(None,arrays.GLdoubleArray)
def glWindowPos3dvARB(v):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glWindowPos3fARB(x,y,z):pass
@_f
@_p.types(None,arrays.GLfloatArray)
def glWindowPos3fvARB(v):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint)
def glWindowPos3iARB(x,y,z):pass
@_f
@_p.types(None,arrays.GLintArray)
def glWindowPos3ivARB(v):pass
@_f
@_p.types(None,_cs.GLshort,_cs.GLshort,_cs.GLshort)
def glWindowPos3sARB(x,y,z):pass
@_f
@_p.types(None,arrays.GLshortArray)
def glWindowPos3svARB(v):pass



'''OpenGL extension NV.vertex_program

This module customises the behaviour of the 
OpenGL.raw.GL.NV.vertex_program to provide a more 
Python-friendly API

Overview (from the spec)
	
	Unextended OpenGL mandates a certain set of configurable per-vertex
	computations defining vertex transformation, texture coordinate
	generation and transformation, and lighting.  Several extensions
	have added further per-vertex computations to OpenGL.  For example,
	extensions have defined new texture coordinate generation modes
	(ARB_texture_cube_map, NV_texgen_reflection, NV_texgen_emboss), new
	vertex transformation modes (EXT_vertex_weighting), new lighting modes
	(OpenGL 1.2's separate specular and rescale normal functionality),
	several modes for fog distance generation (NV_fog_distance), and
	eye-distance point size attenuation (EXT_point_parameters).
	
	Each such extension adds a small set of relatively inflexible
	per-vertex computations.
	
	This inflexibility is in contrast to the typical flexibility provided
	by the underlying programmable floating point engines (whether
	micro-coded vertex engines, DSPs, or CPUs) that are traditionally used
	to implement OpenGL's per-vertex computations.  The purpose of this
	extension is to expose to the OpenGL application writer a significant
	degree of per-vertex programmability for computing vertex parameters.
	
	For the purposes of discussing this extension, a vertex program is
	a sequence of floating-point 4-component vector operations that
	determines how a set of program parameters (defined outside of
	OpenGL's begin/end pair) and an input set of per-vertex parameters
	are transformed to a set of per-vertex output parameters.
	
	The per-vertex computations for standard OpenGL given a particular
	set of lighting and texture coordinate generation modes (along with
	any state for extensions defining per-vertex computations) is, in
	essence, a vertex program.  However, the sequence of operations is
	defined implicitly by the current OpenGL state settings rather than
	defined explicitly as a sequence of instructions.
	
	This extension provides an explicit mechanism for defining vertex
	program instruction sequences for application-defined vertex programs.
	In order to define such vertex programs, this extension defines
	a vertex programming model including a floating-point 4-component
	vector instruction set and a relatively large set of floating-point
	4-component registers.
	
	The extension's vertex programming model is designed for efficient
	hardware implementation and to support a wide variety of vertex
	programs.  By design, the entire set of existing vertex programs
	defined by existing OpenGL per-vertex computation extensions can be
	implemented using the extension's vertex programming model.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/vertex_program.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.vertex_program import *
from OpenGL.raw.GL.NV.vertex_program import _EXTENSION_NAME

def glInitVertexProgramNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glAreProgramsResidentNV.programs size not checked against n
glAreProgramsResidentNV=wrapper.wrapper(glAreProgramsResidentNV).setInputArraySize(
    'programs', None
).setOutput(
    'residences',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
# INPUT glDeleteProgramsNV.programs size not checked against n
glDeleteProgramsNV=wrapper.wrapper(glDeleteProgramsNV).setInputArraySize(
    'programs', None
)
glExecuteProgramNV=wrapper.wrapper(glExecuteProgramNV).setInputArraySize(
    'params', 4
)
glGenProgramsNV=wrapper.wrapper(glGenProgramsNV).setOutput(
    'programs',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
glGetProgramParameterdvNV=wrapper.wrapper(glGetProgramParameterdvNV).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetProgramParameterfvNV=wrapper.wrapper(glGetProgramParameterfvNV).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetProgramivNV=wrapper.wrapper(glGetProgramivNV).setOutput(
    'params',size=(4,),orPassIn=True
)
# OUTPUT glGetProgramStringNV.program COMPSIZE(id, pname) 
glGetTrackMatrixivNV=wrapper.wrapper(glGetTrackMatrixivNV).setOutput(
    'params',size=(1,),orPassIn=True
)
glGetVertexAttribdvNV=wrapper.wrapper(glGetVertexAttribdvNV).setOutput(
    'params',size=(1,),orPassIn=True
)
glGetVertexAttribfvNV=wrapper.wrapper(glGetVertexAttribfvNV).setOutput(
    'params',size=(1,),orPassIn=True
)
glGetVertexAttribivNV=wrapper.wrapper(glGetVertexAttribivNV).setOutput(
    'params',size=(1,),orPassIn=True
)
glGetVertexAttribPointervNV=wrapper.wrapper(glGetVertexAttribPointervNV).setOutput(
    'pointer',size=(1,),orPassIn=True
)
# INPUT glLoadProgramNV.program size not checked against len
glLoadProgramNV=wrapper.wrapper(glLoadProgramNV).setInputArraySize(
    'program', None
)
glProgramParameter4dvNV=wrapper.wrapper(glProgramParameter4dvNV).setInputArraySize(
    'v', 4
)
glProgramParameter4fvNV=wrapper.wrapper(glProgramParameter4fvNV).setInputArraySize(
    'v', 4
)
# INPUT glProgramParameters4dvNV.v size not checked against count*4
glProgramParameters4dvNV=wrapper.wrapper(glProgramParameters4dvNV).setInputArraySize(
    'v', None
)
# INPUT glProgramParameters4fvNV.v size not checked against count*4
glProgramParameters4fvNV=wrapper.wrapper(glProgramParameters4fvNV).setInputArraySize(
    'v', None
)
# INPUT glRequestResidentProgramsNV.programs size not checked against n
glRequestResidentProgramsNV=wrapper.wrapper(glRequestResidentProgramsNV).setInputArraySize(
    'programs', None
)
# INPUT glVertexAttribPointerNV.pointer size not checked against 'fsize,type,stride'
glVertexAttribPointerNV=wrapper.wrapper(glVertexAttribPointerNV).setInputArraySize(
    'pointer', None
)
glVertexAttrib1dvNV=wrapper.wrapper(glVertexAttrib1dvNV).setInputArraySize(
    'v', 1
)
glVertexAttrib1fvNV=wrapper.wrapper(glVertexAttrib1fvNV).setInputArraySize(
    'v', 1
)
glVertexAttrib1svNV=wrapper.wrapper(glVertexAttrib1svNV).setInputArraySize(
    'v', 1
)
glVertexAttrib2dvNV=wrapper.wrapper(glVertexAttrib2dvNV).setInputArraySize(
    'v', 2
)
glVertexAttrib2fvNV=wrapper.wrapper(glVertexAttrib2fvNV).setInputArraySize(
    'v', 2
)
glVertexAttrib2svNV=wrapper.wrapper(glVertexAttrib2svNV).setInputArraySize(
    'v', 2
)
glVertexAttrib3dvNV=wrapper.wrapper(glVertexAttrib3dvNV).setInputArraySize(
    'v', 3
)
glVertexAttrib3fvNV=wrapper.wrapper(glVertexAttrib3fvNV).setInputArraySize(
    'v', 3
)
glVertexAttrib3svNV=wrapper.wrapper(glVertexAttrib3svNV).setInputArraySize(
    'v', 3
)
glVertexAttrib4dvNV=wrapper.wrapper(glVertexAttrib4dvNV).setInputArraySize(
    'v', 4
)
glVertexAttrib4fvNV=wrapper.wrapper(glVertexAttrib4fvNV).setInputArraySize(
    'v', 4
)
glVertexAttrib4svNV=wrapper.wrapper(glVertexAttrib4svNV).setInputArraySize(
    'v', 4
)
glVertexAttrib4ubvNV=wrapper.wrapper(glVertexAttrib4ubvNV).setInputArraySize(
    'v', 4
)
# INPUT glVertexAttribs1dvNV.v size not checked against count
glVertexAttribs1dvNV=wrapper.wrapper(glVertexAttribs1dvNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs1fvNV.v size not checked against count
glVertexAttribs1fvNV=wrapper.wrapper(glVertexAttribs1fvNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs1svNV.v size not checked against count
glVertexAttribs1svNV=wrapper.wrapper(glVertexAttribs1svNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs2dvNV.v size not checked against count*2
glVertexAttribs2dvNV=wrapper.wrapper(glVertexAttribs2dvNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs2fvNV.v size not checked against count*2
glVertexAttribs2fvNV=wrapper.wrapper(glVertexAttribs2fvNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs2svNV.v size not checked against count*2
glVertexAttribs2svNV=wrapper.wrapper(glVertexAttribs2svNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs3dvNV.v size not checked against count*3
glVertexAttribs3dvNV=wrapper.wrapper(glVertexAttribs3dvNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs3fvNV.v size not checked against count*3
glVertexAttribs3fvNV=wrapper.wrapper(glVertexAttribs3fvNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs3svNV.v size not checked against count*3
glVertexAttribs3svNV=wrapper.wrapper(glVertexAttribs3svNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs4dvNV.v size not checked against count*4
glVertexAttribs4dvNV=wrapper.wrapper(glVertexAttribs4dvNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs4fvNV.v size not checked against count*4
glVertexAttribs4fvNV=wrapper.wrapper(glVertexAttribs4fvNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs4svNV.v size not checked against count*4
glVertexAttribs4svNV=wrapper.wrapper(glVertexAttribs4svNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs4ubvNV.v size not checked against count*4
glVertexAttribs4ubvNV=wrapper.wrapper(glVertexAttribs4ubvNV).setInputArraySize(
    'v', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.WGL import _types as _cs
# End users want this...
from OpenGL.raw.WGL._types import *
from OpenGL.raw.WGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'WGL_NV_render_depth_texture'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.WGL,'WGL_NV_render_depth_texture',error_checker=_errors._error_checker)
WGL_BIND_TO_TEXTURE_DEPTH_NV=_C('WGL_BIND_TO_TEXTURE_DEPTH_NV',0x20A3)
WGL_BIND_TO_TEXTURE_RECTANGLE_DEPTH_NV=_C('WGL_BIND_TO_TEXTURE_RECTANGLE_DEPTH_NV',0x20A4)
WGL_DEPTH_COMPONENT_NV=_C('WGL_DEPTH_COMPONENT_NV',0x20A7)
WGL_DEPTH_TEXTURE_FORMAT_NV=_C('WGL_DEPTH_TEXTURE_FORMAT_NV',0x20A5)
WGL_TEXTURE_DEPTH_COMPONENT_NV=_C('WGL_TEXTURE_DEPTH_COMPONENT_NV',0x20A6)




'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_VERSION_GL_2_0'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_VERSION_GL_2_0',error_checker=_errors._error_checker)
GL_ACTIVE_ATTRIBUTES=_C('GL_ACTIVE_ATTRIBUTES',0x8B89)
GL_ACTIVE_ATTRIBUTE_MAX_LENGTH=_C('GL_ACTIVE_ATTRIBUTE_MAX_LENGTH',0x8B8A)
GL_ACTIVE_UNIFORMS=_C('GL_ACTIVE_UNIFORMS',0x8B86)
GL_ACTIVE_UNIFORM_MAX_LENGTH=_C('GL_ACTIVE_UNIFORM_MAX_LENGTH',0x8B87)
GL_ATTACHED_SHADERS=_C('GL_ATTACHED_SHADERS',0x8B85)
GL_BLEND_EQUATION_ALPHA=_C('GL_BLEND_EQUATION_ALPHA',0x883D)
GL_BLEND_EQUATION_RGB=_C('GL_BLEND_EQUATION_RGB',0x8009)
GL_BOOL=_C('GL_BOOL',0x8B56)
GL_BOOL_VEC2=_C('GL_BOOL_VEC2',0x8B57)
GL_BOOL_VEC3=_C('GL_BOOL_VEC3',0x8B58)
GL_BOOL_VEC4=_C('GL_BOOL_VEC4',0x8B59)
GL_COMPILE_STATUS=_C('GL_COMPILE_STATUS',0x8B81)
GL_COORD_REPLACE=_C('GL_COORD_REPLACE',0x8862)
GL_CURRENT_PROGRAM=_C('GL_CURRENT_PROGRAM',0x8B8D)
GL_CURRENT_VERTEX_ATTRIB=_C('GL_CURRENT_VERTEX_ATTRIB',0x8626)
GL_DELETE_STATUS=_C('GL_DELETE_STATUS',0x8B80)
GL_DRAW_BUFFER0=_C('GL_DRAW_BUFFER0',0x8825)
GL_DRAW_BUFFER1=_C('GL_DRAW_BUFFER1',0x8826)
GL_DRAW_BUFFER10=_C('GL_DRAW_BUFFER10',0x882F)
GL_DRAW_BUFFER11=_C('GL_DRAW_BUFFER11',0x8830)
GL_DRAW_BUFFER12=_C('GL_DRAW_BUFFER12',0x8831)
GL_DRAW_BUFFER13=_C('GL_DRAW_BUFFER13',0x8832)
GL_DRAW_BUFFER14=_C('GL_DRAW_BUFFER14',0x8833)
GL_DRAW_BUFFER15=_C('GL_DRAW_BUFFER15',0x8834)
GL_DRAW_BUFFER2=_C('GL_DRAW_BUFFER2',0x8827)
GL_DRAW_BUFFER3=_C('GL_DRAW_BUFFER3',0x8828)
GL_DRAW_BUFFER4=_C('GL_DRAW_BUFFER4',0x8829)
GL_DRAW_BUFFER5=_C('GL_DRAW_BUFFER5',0x882A)
GL_DRAW_BUFFER6=_C('GL_DRAW_BUFFER6',0x882B)
GL_DRAW_BUFFER7=_C('GL_DRAW_BUFFER7',0x882C)
GL_DRAW_BUFFER8=_C('GL_DRAW_BUFFER8',0x882D)
GL_DRAW_BUFFER9=_C('GL_DRAW_BUFFER9',0x882E)
GL_FLOAT_MAT2=_C('GL_FLOAT_MAT2',0x8B5A)
GL_FLOAT_MAT3=_C('GL_FLOAT_MAT3',0x8B5B)
GL_FLOAT_MAT4=_C('GL_FLOAT_MAT4',0x8B5C)
GL_FLOAT_VEC2=_C('GL_FLOAT_VEC2',0x8B50)
GL_FLOAT_VEC3=_C('GL_FLOAT_VEC3',0x8B51)
GL_FLOAT_VEC4=_C('GL_FLOAT_VEC4',0x8B52)
GL_FRAGMENT_SHADER=_C('GL_FRAGMENT_SHADER',0x8B30)
GL_FRAGMENT_SHADER_DERIVATIVE_HINT=_C('GL_FRAGMENT_SHADER_DERIVATIVE_HINT',0x8B8B)
GL_INFO_LOG_LENGTH=_C('GL_INFO_LOG_LENGTH',0x8B84)
GL_INT_VEC2=_C('GL_INT_VEC2',0x8B53)
GL_INT_VEC3=_C('GL_INT_VEC3',0x8B54)
GL_INT_VEC4=_C('GL_INT_VEC4',0x8B55)
GL_LINK_STATUS=_C('GL_LINK_STATUS',0x8B82)
GL_LOWER_LEFT=_C('GL_LOWER_LEFT',0x8CA1)
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS=_C('GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS',0x8B4D)
GL_MAX_DRAW_BUFFERS=_C('GL_MAX_DRAW_BUFFERS',0x8824)
GL_MAX_FRAGMENT_UNIFORM_COMPONENTS=_C('GL_MAX_FRAGMENT_UNIFORM_COMPONENTS',0x8B49)
GL_MAX_TEXTURE_COORDS=_C('GL_MAX_TEXTURE_COORDS',0x8871)
GL_MAX_TEXTURE_IMAGE_UNITS=_C('GL_MAX_TEXTURE_IMAGE_UNITS',0x8872)
GL_MAX_VARYING_FLOATS=_C('GL_MAX_VARYING_FLOATS',0x8B4B)
GL_MAX_VERTEX_ATTRIBS=_C('GL_MAX_VERTEX_ATTRIBS',0x8869)
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS=_C('GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS',0x8B4C)
GL_MAX_VERTEX_UNIFORM_COMPONENTS=_C('GL_MAX_VERTEX_UNIFORM_COMPONENTS',0x8B4A)
GL_POINT_SPRITE=_C('GL_POINT_SPRITE',0x8861)
GL_POINT_SPRITE_COORD_ORIGIN=_C('GL_POINT_SPRITE_COORD_ORIGIN',0x8CA0)
GL_SAMPLER_1D=_C('GL_SAMPLER_1D',0x8B5D)
GL_SAMPLER_1D_SHADOW=_C('GL_SAMPLER_1D_SHADOW',0x8B61)
GL_SAMPLER_2D=_C('GL_SAMPLER_2D',0x8B5E)
GL_SAMPLER_2D_SHADOW=_C('GL_SAMPLER_2D_SHADOW',0x8B62)
GL_SAMPLER_3D=_C('GL_SAMPLER_3D',0x8B5F)
GL_SAMPLER_CUBE=_C('GL_SAMPLER_CUBE',0x8B60)
GL_SHADER_SOURCE_LENGTH=_C('GL_SHADER_SOURCE_LENGTH',0x8B88)
GL_SHADER_TYPE=_C('GL_SHADER_TYPE',0x8B4F)
GL_SHADING_LANGUAGE_VERSION=_C('GL_SHADING_LANGUAGE_VERSION',0x8B8C)
GL_STENCIL_BACK_FAIL=_C('GL_STENCIL_BACK_FAIL',0x8801)
GL_STENCIL_BACK_FUNC=_C('GL_STENCIL_BACK_FUNC',0x8800)
GL_STENCIL_BACK_PASS_DEPTH_FAIL=_C('GL_STENCIL_BACK_PASS_DEPTH_FAIL',0x8802)
GL_STENCIL_BACK_PASS_DEPTH_PASS=_C('GL_STENCIL_BACK_PASS_DEPTH_PASS',0x8803)
GL_STENCIL_BACK_REF=_C('GL_STENCIL_BACK_REF',0x8CA3)
GL_STENCIL_BACK_VALUE_MASK=_C('GL_STENCIL_BACK_VALUE_MASK',0x8CA4)
GL_STENCIL_BACK_WRITEMASK=_C('GL_STENCIL_BACK_WRITEMASK',0x8CA5)
GL_UPPER_LEFT=_C('GL_UPPER_LEFT',0x8CA2)
GL_VALIDATE_STATUS=_C('GL_VALIDATE_STATUS',0x8B83)
GL_VERTEX_ATTRIB_ARRAY_ENABLED=_C('GL_VERTEX_ATTRIB_ARRAY_ENABLED',0x8622)
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED=_C('GL_VERTEX_ATTRIB_ARRAY_NORMALIZED',0x886A)
GL_VERTEX_ATTRIB_ARRAY_POINTER=_C('GL_VERTEX_ATTRIB_ARRAY_POINTER',0x8645)
GL_VERTEX_ATTRIB_ARRAY_SIZE=_C('GL_VERTEX_ATTRIB_ARRAY_SIZE',0x8623)
GL_VERTEX_ATTRIB_ARRAY_STRIDE=_C('GL_VERTEX_ATTRIB_ARRAY_STRIDE',0x8624)
GL_VERTEX_ATTRIB_ARRAY_TYPE=_C('GL_VERTEX_ATTRIB_ARRAY_TYPE',0x8625)
GL_VERTEX_PROGRAM_POINT_SIZE=_C('GL_VERTEX_PROGRAM_POINT_SIZE',0x8642)
GL_VERTEX_PROGRAM_TWO_SIDE=_C('GL_VERTEX_PROGRAM_TWO_SIDE',0x8643)
GL_VERTEX_SHADER=_C('GL_VERTEX_SHADER',0x8B31)
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint)
def glAttachShader(program,shader):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,arrays.GLcharArray)
def glBindAttribLocation(program,index,name):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum)
def glBlendEquationSeparate(modeRGB,modeAlpha):pass
@_f
@_p.types(None,_cs.GLuint)
def glCompileShader(shader):pass
@_f
@_p.types(_cs.GLuint,)
def glCreateProgram():pass
@_f
@_p.types(_cs.GLuint,_cs.GLenum)
def glCreateShader(type):pass
@_f
@_p.types(None,_cs.GLuint)
def glDeleteProgram(program):pass
@_f
@_p.types(None,_cs.GLuint)
def glDeleteShader(shader):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint)
def glDetachShader(program,shader):pass
@_f
@_p.types(None,_cs.GLuint)
def glDisableVertexAttribArray(index):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glDrawBuffers(n,bufs):pass
@_f
@_p.types(None,_cs.GLuint)
def glEnableVertexAttribArray(index):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLintArray,arrays.GLuintArray,arrays.GLcharArray)
def glGetActiveAttrib(program,index,bufSize,length,size,type,name):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLintArray,arrays.GLuintArray,arrays.GLcharArray)
def glGetActiveUniform(program,index,bufSize,length,size,type,name):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLuintArray)
def glGetAttachedShaders(program,maxCount,count,shaders):pass
@_f
@_p.types(_cs.GLint,_cs.GLuint,arrays.GLcharArray)
def glGetAttribLocation(program,name):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetProgramInfoLog(program,bufSize,length,infoLog):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetProgramiv(program,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetShaderInfoLog(shader,bufSize,length,infoLog):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetShaderSource(shader,bufSize,length,source):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetShaderiv(shader,pname,params):pass
@_f
@_p.types(_cs.GLint,_cs.GLuint,arrays.GLcharArray)
def glGetUniformLocation(program,name):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,arrays.GLfloatArray)
def glGetUniformfv(program,location,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,arrays.GLintArray)
def glGetUniformiv(program,location,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLvoidpArray)
def glGetVertexAttribPointerv(index,pname,pointer):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLdoubleArray)
def glGetVertexAttribdv(index,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLfloatArray)
def glGetVertexAttribfv(index,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetVertexAttribiv(index,pname,params):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glIsProgram(program):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glIsShader(shader):pass
@_f
@_p.types(None,_cs.GLuint)
def glLinkProgram(program):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,ctypes.POINTER( ctypes.POINTER( _cs.GLchar )),arrays.GLintArray)
def glShaderSource(shader,count,string,length):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLuint)
def glStencilFuncSeparate(face,func,ref,mask):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glStencilMaskSeparate(face,mask):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLenum)
def glStencilOpSeparate(face,sfail,dpfail,dppass):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLfloat)
def glUniform1f(location,v0):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLfloatArray)
def glUniform1fv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint)
def glUniform1i(location,v0):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLintArray)
def glUniform1iv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLfloat,_cs.GLfloat)
def glUniform2f(location,v0,v1):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLfloatArray)
def glUniform2fv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint)
def glUniform2i(location,v0,v1):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLintArray)
def glUniform2iv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glUniform3f(location,v0,v1,v2):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLfloatArray)
def glUniform3fv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint)
def glUniform3i(location,v0,v1,v2):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLintArray)
def glUniform3iv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glUniform4f(location,v0,v1,v2,v3):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLfloatArray)
def glUniform4fv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint)
def glUniform4i(location,v0,v1,v2,v3):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLintArray)
def glUniform4iv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glUniformMatrix2fv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glUniformMatrix3fv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glUniformMatrix4fv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint)
def glUseProgram(program):pass
@_f
@_p.types(None,_cs.GLuint)
def glValidateProgram(program):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLdouble)
def glVertexAttrib1d(index,x):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLdoubleArray)
def glVertexAttrib1dv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLfloat)
def glVertexAttrib1f(index,x):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLfloatArray)
def glVertexAttrib1fv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLshort)
def glVertexAttrib1s(index,x):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLshortArray)
def glVertexAttrib1sv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLdouble,_cs.GLdouble)
def glVertexAttrib2d(index,x,y):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLdoubleArray)
def glVertexAttrib2dv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLfloat,_cs.GLfloat)
def glVertexAttrib2f(index,x,y):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLfloatArray)
def glVertexAttrib2fv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLshort,_cs.GLshort)
def glVertexAttrib2s(index,x,y):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLshortArray)
def glVertexAttrib2sv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glVertexAttrib3d(index,x,y,z):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLdoubleArray)
def glVertexAttrib3dv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glVertexAttrib3f(index,x,y,z):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLfloatArray)
def glVertexAttrib3fv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLshort,_cs.GLshort,_cs.GLshort)
def glVertexAttrib3s(index,x,y,z):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLshortArray)
def glVertexAttrib3sv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLbyteArray)
def glVertexAttrib4Nbv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLintArray)
def glVertexAttrib4Niv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLshortArray)
def glVertexAttrib4Nsv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLubyte,_cs.GLubyte,_cs.GLubyte,_cs.GLubyte)
def glVertexAttrib4Nub(index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLubyteArray)
def glVertexAttrib4Nubv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLuintArray)
def glVertexAttrib4Nuiv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLushortArray)
def glVertexAttrib4Nusv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLbyteArray)
def glVertexAttrib4bv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glVertexAttrib4d(index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLdoubleArray)
def glVertexAttrib4dv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glVertexAttrib4f(index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLfloatArray)
def glVertexAttrib4fv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLintArray)
def glVertexAttrib4iv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLshort,_cs.GLshort,_cs.GLshort,_cs.GLshort)
def glVertexAttrib4s(index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLshortArray)
def glVertexAttrib4sv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLubyteArray)
def glVertexAttrib4ubv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLuintArray)
def glVertexAttrib4uiv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLushortArray)
def glVertexAttrib4usv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLboolean,_cs.GLsizei,ctypes.c_void_p)
def glVertexAttribPointer(index,size,type,normalized,stride,pointer):pass



'''OpenGL extension SGIX.framezoom

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.framezoom to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a additional way to rasterize geometric
	primitives and pixel rectangles.  The techique is to reduce the
	number of pixels rasterized  and (possibly) the number of depth and
	stencil operations performed per primitive.  Each pixel is zoomed
	up and used to render an N x N block of screen pixels.  The
	implementation is free to choose the number of stencil and z pixels
	that will correspond to each N x N block.
	
	This extension provides an opportunity to the implementation to
	perform expensive raster operations at a reduced resolution,
	increasing performance.  Such operations may include
	texture-mapping, depth & stencil tests, etc.  The hardware should
	be allowed to perform operations that it accelerates at full
	hardware speed.
	
	The visual result will be the same as if a scene were rendered into
	a small window, and then that buffer was copied and zoomed up into
	a large window.
	
	All OpenGL parameters that effect rasterization size will implicitly
	be multipled by N (this includes point size, line width, etc).

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/framezoom.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.framezoom import *
from OpenGL.raw.GL.SGIX.framezoom import _EXTENSION_NAME

def glInitFramezoomSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_AMD_sparse_texture'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_AMD_sparse_texture',error_checker=_errors._error_checker)
GL_MAX_SPARSE_3D_TEXTURE_SIZE_AMD=_C('GL_MAX_SPARSE_3D_TEXTURE_SIZE_AMD',0x9199)
GL_MAX_SPARSE_ARRAY_TEXTURE_LAYERS=_C('GL_MAX_SPARSE_ARRAY_TEXTURE_LAYERS',0x919A)
GL_MAX_SPARSE_TEXTURE_SIZE_AMD=_C('GL_MAX_SPARSE_TEXTURE_SIZE_AMD',0x9198)
GL_MIN_LOD_WARNING_AMD=_C('GL_MIN_LOD_WARNING_AMD',0x919C)
GL_MIN_SPARSE_LEVEL_AMD=_C('GL_MIN_SPARSE_LEVEL_AMD',0x919B)
GL_TEXTURE_STORAGE_SPARSE_BIT_AMD=_C('GL_TEXTURE_STORAGE_SPARSE_BIT_AMD',0x00000001)
GL_VIRTUAL_PAGE_SIZE_X_AMD=_C('GL_VIRTUAL_PAGE_SIZE_X_AMD',0x9195)
GL_VIRTUAL_PAGE_SIZE_Y_AMD=_C('GL_VIRTUAL_PAGE_SIZE_Y_AMD',0x9196)
GL_VIRTUAL_PAGE_SIZE_Z_AMD=_C('GL_VIRTUAL_PAGE_SIZE_Z_AMD',0x9197)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLbitfield)
def glTexStorageSparseAMD(target,internalFormat,width,height,depth,layers,flags):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLbitfield)
def glTextureStorageSparseAMD(texture,target,internalFormat,width,height,depth,layers,flags):pass
